/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref;

import java.awt.Desktop;
import java.awt.GraphicsEnvironment;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

import javax.swing.SwingUtilities;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.emse.gitlab.saref.gui.MainFrame;

/**
 *
 * @author Maxime Lefrançois
 *
 */
public class Main {

	private static final Logger LOG = LoggerFactory.getLogger(Main.class);

	public static void main(String[] args) {
		if (args.length == 0 && !GraphicsEnvironment.isHeadless()) {
			mainGUI();
		} else {
			mainCLI(args);
		} 
	}

	private static void mainCLI(String[] args) {
		CLIExecution cliExecution = new CLIExecution();
		final SAREFPipeline pipeline = cliExecution.getSAREFPipeline(args);
		try {
			boolean reportGenerated = pipeline.run();
			if (reportGenerated) {
				try {	
					final File reportHTML = new File(pipeline.targetDir, "site/report.html");
					if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.BROWSE)) {
						Desktop.getDesktop().browse(reportHTML.toURI());
					} else {
						System.out.println("\n\n\nURL to the SAREF pipeline report:\n" + reportHTML.getCanonicalPath());
					}
				} catch (IOException ex) {
					throw new SAREFPipelineException(ex);
				}
			}
		} catch (SAREFPipelineException ex) {
			LOG.error("Exception:", ex);
			System.exit(-1);
		}
	}

	private static void mainGUI() {
		try {
			String info = IOUtils.toString(Main.class.getClassLoader().getResourceAsStream("cli/info.txt"),
					StandardCharsets.UTF_8);
			System.out.println(info);
			MainFrame application = new MainFrame();
			SwingUtilities.updateComponentTreeUI(application);
			java.awt.EventQueue.invokeLater(new Runnable() {
				@Override
				public void run() {
					application.toFront();
					application.repaint();
				}
			});
		} catch (Exception ex) {
			LOG.warn("Exception ", ex);
			System.exit(-1);
		}
	}

}
